@@ -1,7 +1,13 @@
+# frozen-string-literal: true
+
 Sequel.require 'adapters/shared/postgres'
 
 begin 
   require 'pg' 
+
+  # Work around postgres-pr 0.7.0+ which ships with a pg.rb file
+  raise LoadError unless defined?(PGconn::CONNECTION_OK)
+
   SEQUEL_POSTGRES_USES_PG = true
 rescue LoadError => e 
   SEQUEL_POSTGRES_USES_PG = false
@@ -108,7 +114,25 @@ module Sequel
     # PGconn subclass for connection specific methods used with the
     # pg, postgres, or postgres-pr driver.
     class Adapter < ::PGconn
-      DISCONNECT_ERROR_RE = /\A(?:could not receive data from server|no connection to the server|connection not open|terminating connection due to administrator command)/
+      # The underlying exception classes to reraise as disconnect errors
+      # instead of regular database errors.
+      DISCONNECT_ERROR_CLASSES = [IOError, Errno::EPIPE, Errno::ECONNRESET]
+      if defined?(::PG::ConnectionBad)
+        DISCONNECT_ERROR_CLASSES << ::PG::ConnectionBad
+      end
+      
+      disconnect_errors = [
+        'could not receive data from server',
+        'no connection to the server',
+        'connection not open',
+        'terminating connection due to administrator command',
+        'PQconsumeInput() '
+       ]
+
+      # Since exception class based disconnect checking may not work,
+      # also trying parsing the exception message to look for disconnect
+      # errors.
+      DISCONNECT_ERROR_RE = /\A#{Regexp.union(disconnect_errors)}/
       
       self.translate_results = false if respond_to?(:translate_results=)
       
@@ -117,11 +141,15 @@ module Sequel
       # are SQL strings.
       attr_reader(:prepared_statements) if SEQUEL_POSTGRES_USES_PG
       
-      # Raise a Sequel::DatabaseDisconnectError if a PGError is raised and
-      # the connection status cannot be determined or it is not OK.
+      # Raise a Sequel::DatabaseDisconnectError if a one of the disconnect
+      # error classes is raised, or a PGError is raised and the connection
+      # status cannot be determined or it is not OK.
       def check_disconnect_errors
         begin
           yield
+        rescue *DISCONNECT_ERROR_CLASSES => e
+          disconnect = true
+          raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError))
         rescue PGError => e
           disconnect = false
           begin
@@ -133,9 +161,6 @@ module Sequel
           disconnect ||= !status_ok
           disconnect ||= e.message =~ DISCONNECT_ERROR_RE
           disconnect ? raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError)) : raise
-        rescue IOError, Errno::EPIPE, Errno::ECONNRESET => e
-          disconnect = true
-          raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError))
         ensure
           block if status_ok && !disconnect
         end
@@ -170,6 +195,7 @@ module Sequel
       INFINITE_TIMESTAMP_STRINGS = ['infinity'.freeze, '-infinity'.freeze].freeze
       INFINITE_DATETIME_VALUES = ([PLUS_INFINITY, MINUS_INFINITY] + INFINITE_TIMESTAMP_STRINGS).freeze
       
+      set_adapter_scheme :postgresql
       set_adapter_scheme :postgres
 
       # Whether infinite timestamps/dates should be converted on retrieval.  By default, no
@@ -185,7 +211,7 @@ module Sequel
       def bound_variable_arg(arg, conn)
         case arg
         when Sequel::SQL::Blob
-          conn.escape_bytea(arg)
+          {:value=>arg, :type=>17, :format=>1}
         when Sequel::SQLTime
           literal(arg)
         when DateTime, Time
@@ -351,7 +377,7 @@ module Sequel
                 end
                 nil
               else
-                b = ''
+                b = String.new
                 b << buf while buf = conn.get_copy_data
                 b
               end
@@ -419,14 +445,16 @@ module Sequel
         # :after_listen :: An object that responds to +call+ that is called with the underlying connection after the LISTEN
         #                  statement is sent, but before the connection starts waiting for notifications.
         # :loop :: Whether to continually wait for notifications, instead of just waiting for a single
-        #          notification. If this option is given, a block must be provided.  If this object responds to call, it is
+        #          notification. If this option is given, a block must be provided.  If this object responds to +call+, it is
         #          called with the underlying connection after each notification is received (after the block is called).
         #          If a :timeout option is used, and a callable object is given, the object will also be called if the
         #          timeout expires.  If :loop is used and you want to stop listening, you can either break from inside the
         #          block given to #listen, or you can throw :stop from inside the :loop object's call method or the block.
         # :server :: The server on which to listen, if the sharding support is being used.
-        # :timeout :: How long to wait for a notification, in seconds (can provide a float value for
-        #             fractional seconds).  If not given or nil, waits indefinitely.
+        # :timeout :: How long to wait for a notification, in seconds (can provide a float value for fractional seconds).
+        #             If this object responds to +call+, it will be called and should return the number of seconds to wait.
+        #             If the loop option is also specified, the object will be called on each iteration to obtain a new
+        #             timeout value.  If not given or nil, waits indefinitely.
         #
         # This method is only supported if pg is used as the underlying ruby driver.  It returns the
         # channel the notification was sent to (as a string), unless :loop was used, in which case it returns nil.
@@ -440,24 +468,30 @@ module Sequel
               begin
                 channels = Array(channels)
                 channels.each do |channel|
-                  sql = "LISTEN "
+                  sql = "LISTEN ".dup
                   dataset.send(:identifier_append, sql, channel)
                   conn.execute(sql)
                 end
                 opts[:after_listen].call(conn) if opts[:after_listen]
-                timeout = opts[:timeout] ? [opts[:timeout]] : []
+                timeout = opts[:timeout]
+                if timeout
+                  timeout_block = timeout.respond_to?(:call) ? timeout : proc{timeout}
+                end
+
                 if l = opts[:loop]
                   raise Error, 'calling #listen with :loop requires a block' unless block
                   loop_call = l.respond_to?(:call)
                   catch(:stop) do
                     loop do
-                      conn.wait_for_notify(*timeout, &block)
+                      t = timeout_block ? [timeout_block.call] : []
+                      conn.wait_for_notify(*t, &block)
                       l.call(conn) if loop_call
                     end
                   end
                   nil
                 else
-                  conn.wait_for_notify(*timeout, &block)
+                  t = timeout_block ? [timeout_block.call] : []
+                  conn.wait_for_notify(*t, &block)
                 end
               ensure
                 conn.execute("UNLISTEN *")
@@ -557,7 +591,7 @@ module Sequel
 
         log_sql = "EXECUTE #{ps_name}"
         if ps.log_sql
-          log_sql << " ("
+          log_sql += " ("
           log_sql << sql
           log_sql << ")"
         end
@@ -642,9 +676,11 @@ module Sequel
       end
 
       # Uses a cursor for fetching records, instead of fetching the entire result
-      # set at once.  Can be used to process large datasets without holding
-      # all rows in memory (which is what the underlying drivers may do
-      # by default). Options:
+      # set at once.  Note this uses a transaction around the cursor usage by
+      # default and can be changed using `hold: true` as described below.
+      # Cursors can be used to process large datasets without holding all rows
+      # in memory (which is what the underlying drivers may do by default).
+      # Options:
       #
       # :cursor_name :: The name assigned to the cursor (default 'sequel_cursor').
       #                 Nested cursors require different names.
@@ -662,7 +698,7 @@ module Sequel
       # This is untested with the prepared statement/bound variable support,
       # and unlikely to work with either.
       def use_cursor(opts=OPTS)
-        clone(:cursor=>{:rows_per_fetch=>1000}.merge(opts))
+        clone(:cursor=>{:rows_per_fetch=>1000}.merge!(opts))
       end
 
       # Replace the WHERE clause with one that uses CURRENT OF with the given
@@ -713,47 +749,14 @@ module Sequel
           end
         end
 
-        # Allow use of bind arguments for PostgreSQL using the pg driver.
-        module BindArgumentMethods
-          include ArgumentMapper
-          include ::Sequel::Postgres::DatasetMethods::PreparedStatementMethods
-          
-          private
-          
-          # Execute the given SQL with the stored bind arguments.
-          def execute(sql, opts=OPTS, &block)
-            super(sql, {:arguments=>bind_arguments}.merge(opts), &block)
-          end
-          
-          # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts=OPTS, &block)
-            super(sql, {:arguments=>bind_arguments}.merge(opts), &block)
-          end
-        end
-        
-        # Allow use of server side prepared statements for PostgreSQL using the
-        # pg driver.
-        module PreparedStatementMethods
-          include BindArgumentMethods
+        BindArgumentMethods = prepared_statements_module(:bind, [ArgumentMapper, ::Sequel::Postgres::DatasetMethods::PreparedStatementMethods], %w'execute execute_dui')
 
+        PreparedStatementMethods = prepared_statements_module(:prepare, BindArgumentMethods, %w'execute execute_dui') do
           # Raise a more obvious error if you attempt to call a unnamed prepared statement.
           def call(*)
             raise Error, "Cannot call prepared statement without a name" if prepared_statement_name.nil?
             super
           end
-          
-          private
-          
-          # Execute the stored prepared statement name and the stored bind
-          # arguments instead of the SQL given.
-          def execute(sql, opts=OPTS, &block)
-            super(prepared_statement_name, opts, &block)
-          end
-          
-          # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts=OPTS, &block)
-            super(prepared_statement_name, opts, &block)
-          end
         end
         
         # Execute the given type of statement with the hash of values.
