@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+
 Sequel.require 'adapters/utils/split_alter_table'
 Sequel.require 'adapters/utils/replace'
 
@@ -185,58 +187,64 @@ module Sequel
       
       private
       
-      # Use MySQL specific syntax for some alter table operations.
-      def alter_table_op_sql(table, op)
-        case op[:op]
-        when :add_column
-          if related = op.delete(:table)
-            sql = super
-            op[:table] = related
-            op[:key] ||= primary_key_from_schema(related)
-            sql << ", ADD "
-            if constraint_name = op.delete(:foreign_key_constraint_name)
-              sql << "CONSTRAINT #{quote_identifier(constraint_name)} "
-            end
-            sql << "FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"
-          else
-            super
+      def alter_table_add_column_sql(table, op)
+        if related = op.delete(:table)
+          sql = super.dup
+          op[:table] = related
+          op[:key] ||= primary_key_from_schema(related)
+          sql << ", ADD "
+          if constraint_name = op.delete(:foreign_key_constraint_name)
+            sql << "CONSTRAINT #{quote_identifier(constraint_name)} "
           end
-        when :rename_column, :set_column_type, :set_column_null, :set_column_default
-          o = op[:op]
-          opts = schema(table).find{|x| x.first == op[:name]}
-          opts = opts ? opts.last.dup : {}
-          opts[:name] = o == :rename_column ? op[:new_name] : op[:name]
-          opts[:type] = o == :set_column_type ? op[:type] : opts[:db_type]
-          opts[:null] = o == :set_column_null ? op[:null] : opts[:allow_null]
-          opts[:default] = o == :set_column_default ? op[:default] : opts[:ruby_default]
-          opts.delete(:default) if opts[:default] == nil
-          opts.delete(:primary_key)
-          unless op[:type] || opts[:type]
-            raise Error, "cannot determine database type to use for CHANGE COLUMN operation"
-          end
-          opts = op.merge(opts)
-          opts.delete(:auto_increment) if op[:auto_increment] == false
-          "CHANGE COLUMN #{quote_identifier(op[:name])} #{column_definition_sql(opts)}"
-        when :drop_constraint
-          case op[:type]
-          when :primary_key
-            "DROP PRIMARY KEY"
-          when :foreign_key
-            name = op[:name] || foreign_key_name(table, op[:columns])
-            "DROP FOREIGN KEY #{quote_identifier(name)}"
-          when :unique
-            "DROP INDEX #{quote_identifier(op[:name])}"
-          end
-        when :add_constraint
-          if op[:type] == :foreign_key
-            op[:key] ||= primary_key_from_schema(op[:table])
-          end
-          super
+          sql << "FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"
         else
           super
         end
       end
 
+      def alter_table_change_column_sql(table, op)
+        o = op[:op]
+        opts = schema(table).find{|x| x.first == op[:name]}
+        opts = opts ? opts.last.dup : {}
+        opts[:name] = o == :rename_column ? op[:new_name] : op[:name]
+        opts[:type] = o == :set_column_type ? op[:type] : opts[:db_type]
+        opts[:null] = o == :set_column_null ? op[:null] : opts[:allow_null]
+        opts[:default] = o == :set_column_default ? op[:default] : opts[:ruby_default]
+        opts.delete(:default) if opts[:default] == nil
+        opts.delete(:primary_key)
+        unless op[:type] || opts[:type]
+          raise Error, "cannot determine database type to use for CHANGE COLUMN operation"
+        end
+        opts = op.merge(opts)
+        if op.has_key?(:auto_increment)
+          opts[:auto_increment] = op[:auto_increment]
+        end
+        "CHANGE COLUMN #{quote_identifier(op[:name])} #{column_definition_sql(opts)}"
+      end
+      alias alter_table_rename_column_sql alter_table_change_column_sql
+      alias alter_table_set_column_type_sql alter_table_change_column_sql
+      alias alter_table_set_column_null_sql alter_table_change_column_sql
+      alias alter_table_set_column_default_sql alter_table_change_column_sql
+
+      def alter_table_add_constraint_sql(table, op)
+        if op[:type] == :foreign_key
+          op[:key] ||= primary_key_from_schema(op[:table])
+        end
+        super
+      end
+
+      def alter_table_drop_constraint_sql(table, op)
+        case op[:type]
+        when :primary_key
+          "DROP PRIMARY KEY"
+        when :foreign_key
+          name = op[:name] || foreign_key_name(table, op[:columns])
+          "DROP FOREIGN KEY #{quote_identifier(name)}"
+        when :unique
+          "DROP INDEX #{quote_identifier(op[:name])}"
+        end
+      end
+
       # MySQL server requires table names when dropping indexes.
       def alter_table_sql(table, op)
         case op[:op]
@@ -274,9 +282,11 @@ module Sequel
       def mysql_connection_setting_sqls
         sqls = []
         
-        # Increase timeout so mysql server doesn't disconnect us
-        # Value used by default is maximum allowed value on Windows.
-        sqls << "SET @@wait_timeout = #{opts[:timeout] || 2147483}"
+        if wait_timeout = opts.fetch(:timeout, 2147483)
+          # Increase timeout so mysql server doesn't disconnect us
+          # Value used by default is maximum allowed value on Windows.
+          sqls << "SET @@wait_timeout = #{wait_timeout}"
+        end
 
         # By default, MySQL 'where id is null' selects the last inserted id
         sqls <<  "SET SQL_AUTO_IS_NULL=0" unless opts[:auto_is_null]
@@ -348,6 +358,8 @@ module Sequel
               [pk]
             elsif !(pkc = generator.constraints.select{|con| con[:type] == :primary_key}).empty?
               pkc.first[:columns]
+            elsif !(pkc = generator.columns.select{|con| con[:primary_key] == true}).empty?
+              pkc.map{|c| c[:name]}
             end
           else
             primary_key_from_schema(t)
@@ -460,11 +472,12 @@ module Sequel
         table = SQL::Identifier.new(im.call(table_name))
         table = SQL::QualifiedIdentifier.new(im.call(opts[:schema]), table) if opts[:schema]
         metadata_dataset.with_sql("DESCRIBE ?", table).map do |row|
-          row[:auto_increment] = true if row.delete(:Extra).to_s =~ /auto_increment/io
+          extra = row.delete(:Extra)
+          if row[:primary_key] = row.delete(:Key) == 'PRI'
+            row[:auto_increment] = !!(extra.to_s =~ /auto_increment/io)
+          end
           row[:allow_null] = row.delete(:Null) == 'YES'
           row[:default] = row.delete(:Default)
-          row[:primary_key] = row.delete(:Key) == 'PRI'
-          row[:default] = nil if blank_object?(row[:default])
           row[:db_type] = row.delete(:Type)
           row[:type] = schema_column_type(row[:db_type])
           [m.call(row.delete(:Field)), row]
