@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+#
 # The constraint_validations extension is designed to easily create database
 # constraints inside create_table and alter_table blocks.  It also adds
 # relevant metadata about the constraints to a separate table, which the
@@ -146,7 +148,7 @@ module Sequel
       %w'presence unique'.each do |v|
         class_eval(<<-END, __FILE__, __LINE__+1)
           def #{v}(columns, opts=OPTS)
-            @generator.validation({:type=>:#{v}, :columns=>Array(columns)}.merge(opts))
+            @generator.validation({:type=>:#{v}, :columns=>Array(columns)}.merge!(opts))
           end
         END
       end
@@ -155,7 +157,7 @@ module Sequel
       %w'exact_length min_length max_length length_range format like ilike includes'.each do |v|
         class_eval(<<-END, __FILE__, __LINE__+1)
           def #{v}(arg, columns, opts=OPTS)
-            @generator.validation({:type=>:#{v}, :columns=>Array(columns), :arg=>arg}.merge(opts))
+            @generator.validation({:type=>:#{v}, :columns=>Array(columns), :arg=>arg}.merge!(opts))
           end
         END
       end
@@ -224,6 +226,16 @@ module Sequel
       end
     end
 
+    # Modify the default create_table generator to include
+    # the constraint validation methods.
+    def create_table_generator(&block)
+      super do
+        extend CreateTableGeneratorMethods
+        @validations = []
+        instance_eval(&block) if block
+      end
+    end
+
     # Drop the constraint validations table.
     def drop_constraint_validations_table
       drop_table(constraint_validations_table)
@@ -262,9 +274,7 @@ module Sequel
       ds.delete
     end
 
-    private
-
-    # Modify the default create_table generator to include
+    # Modify the default alter_table generator to include
     # the constraint validation methods.
     def alter_table_generator(&block)
       super do
@@ -274,6 +284,8 @@ module Sequel
       end
     end
 
+    private
+
     # After running all of the table alteration statements,
     # if there were any constraint validations, run table alteration
     # statements to create related constraints.  This is purposely
@@ -317,16 +329,6 @@ module Sequel
       super
     end
 
-    # Modify the default create_table generator to include
-    # the constraint validation methods.
-    def create_table_generator(&block)
-      super do
-        extend CreateTableGeneratorMethods
-        @validations = []
-        instance_eval(&block) if block
-      end
-    end
-
     # For the given table, generator, and validations, add constraints
     # to the generator for each of the validations, as well as adding
     # validation metadata to the constraint validations table.
