@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+#
 # Adds the Sequel::Migration and Sequel::Migrator classes, which allow
 # the user to easily group schema changes and migrate the database
 # to a newer version or revert to a previous version.
@@ -203,12 +205,12 @@ module Sequel
       @actions << [:drop_join_table, *args]
     end
 
-    def create_table(*args)
-      @actions << [:drop_table, args.first]
+    def create_table(name, opts=OPTS)
+      @actions << [:drop_table, name, opts]
     end
 
-    def create_view(name, _, options={})
-      @actions << [:drop_view, name, options]
+    def create_view(name, _, opts=OPTS)
+      @actions << [:drop_view, name, opts]
     end
 
     def rename_column(table, name, new_name)
@@ -467,6 +469,14 @@ module Sequel
       end
     end
 
+    # Load the migration file, raising an exception if the file does not define
+    # a single migration.
+    def load_migration_file(file)
+      n = Migration.descendants.length
+      load(file)
+      raise Error, "Migration file not containing a single migration detected" unless n + 1 == Migration.descendants.length
+    end
+
     # Remove all migration classes.  Done by the migrator to ensure that
     # the correct migration classes are picked up.
     def remove_migration_classes
@@ -523,13 +533,12 @@ module Sequel
     def run
       migrations.zip(version_numbers).each do |m, v|
         t = Time.now
-        lv = up? ? v : v + 1
-        db.log_info("Begin applying migration version #{lv}, direction: #{direction}")
+        db.log_info("Begin applying migration version #{v}, direction: #{direction}")
         checked_transaction(m) do
           m.apply(db, direction)
-          set_migration_version(v)
+          set_migration_version(up? ? v : v-1)
         end
-        db.log_info("Finished applying migration version #{lv}, direction: #{direction}, took #{sprintf('%0.6f', Time.now - t)} seconds")
+        db.log_info("Finished applying migration version #{v}, direction: #{direction}, took #{sprintf('%0.6f', Time.now - t)} seconds")
       end
       
       target
@@ -565,11 +574,10 @@ module Sequel
       remove_migration_classes
 
       # load migration files
-      files[up? ? (current + 1)..target : (target + 1)..current].compact.each{|f| load(f)}
+      version_numbers.each{|n| load_migration_file(files[n])}
       
       # get migration classes
-      classes = Migration.descendants
-      up? ? classes : classes.reverse
+      Migration.descendants
     end
     
     # Returns the latest version available in the specified directory.
@@ -606,7 +614,13 @@ module Sequel
     # so that each number in the array is the migration version
     # that will be in affect after the migration is run.
     def version_numbers
-      up? ? ((current+1)..target).to_a : (target..(current - 1)).to_a.reverse
+      versions = files.
+        compact.
+        map{|f| migration_version_from_file(File.basename(f))}.
+        select{|v| up? ? (v > current && v <= target) : (v <= current && v > target)}.
+        sort
+      versions.reverse! unless up?
+      versions
     end
   end
 
@@ -701,15 +715,15 @@ module Sequel
         if target
           if migration_version_from_file(f) > target
             if applied_migrations.include?(fi)
-              load(path)
+              load_migration_file(path)
               down_mts << [ms.last, f, :down]
             end
           elsif !applied_migrations.include?(fi)
-            load(path)
+            load_migration_file(path)
             up_mts << [ms.last, f, :up]
           end
         elsif !applied_migrations.include?(fi)
-          load(path)
+          load_migration_file(path)
           up_mts << [ms.last, f, :up]
         end
       end
