@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+#
 # The pg_hstore extension adds support for the PostgreSQL hstore type
 # to Sequel.  hstore is an extension that ships with PostgreSQL, and
 # the hstore type stores an arbitrary key-value table, where the keys
@@ -8,7 +10,7 @@
 # as instances of Sequel::Postgres::HStore.  HStore is
 # a DelegateClass of Hash, so it mostly acts like a hash, but not
 # completely (is_a?(Hash) is false).  If you want the actual hash,
-# you can call Hstore#to_hash.  This is done so that Sequel does not
+# you can call HStore#to_hash.  This is done so that Sequel does not
 # treat a HStore like a Hash by default, which would cause issues.
 #
 # In addition to the parsers, this extension comes with literalizers
@@ -141,7 +143,7 @@ module Sequel
       module DatabaseMethods
         def self.extended(db)
           db.instance_eval do
-            add_named_conversion_procs(conversion_procs, :hstore=>PG_NAMED_TYPES[:hstore])
+            add_named_conversion_proc(:hstore)
             @schema_type_classes[:hstore] = HStore
           end
         end
@@ -265,7 +267,7 @@ module Sequel
       # literal version of the hstore.  Separated out for use by
       # the bound argument code.
       def unquoted_literal
-        str = ''
+        str = String.new
         comma = false
         commas = COMMA
         quote = QUOTE
