@@ -1,7 +1,9 @@
+# frozen-string-literal: true
+#
 # The pg_row extension adds support for Sequel to handle
 # PostgreSQL's row-valued/composite types.
 #
-# This extension integrates with Sequel's native postgres adapter, so
+# This extension integrates with Sequel's native postgres and jdbc/postgresql adapters, so
 # that when composite fields are retrieved, they are parsed and returned
 # as instances of Sequel::Postgres::PGRow::(HashRow|ArrayRow), or
 # optionally a custom type.  HashRow and ArrayRow are DelegateClasses of
@@ -27,8 +29,8 @@
 #
 #   Sequel.pg_row(array)
 #
-# If you have loaded the {core_extensions extension}[link:files/doc/core_extensions_rdoc.html]),
-# or you have loaded the {core_refinements extension}[link:files/doc/core_refinements_rdoc.html])
+# If you have loaded the {core_extensions extension}[rdoc-ref:doc/core_extensions.rdoc],
+# or you have loaded the core_refinements extension
 # and have activated refinements for the file, you can also use Array#pg_row:
 #
 #   array.pg_row
@@ -74,9 +76,12 @@
 #   DB.conversion_procs.select{|k,v| v.is_a?(Sequel::Postgres::PGRow::Parser) && \
 #     v.converter && (v.converter.name.nil? || v.converter.name == '') }.map{|k,v| v}
 # 
-# If you are not using the native postgres adapter, you probably
-# also want to use the pg_typecast_on_load plugin in the model, and
-# set it to typecast the composite type column(s) on load.
+# If you are not using the native postgres or jdbc/postgresql adapters and are using composite types
+# types as model column values you probably should use the
+# pg_typecast_on_load plugin if the column values are returned as a string.
+#
+# See the {schema modification guide}[rdoc-ref:doc/schema_modification.rdoc]
+# for details on using row type columns in CREATE/ALTER TABLE statements.
 #
 # This extension requires both the strscan and delegate libraries.
 
@@ -442,7 +447,7 @@ module Sequel
           end
           # Manually cast to integer using to_i, because adapter may not cast oid type
           # correctly (e.g. swift)
-          parser_opts[:oid], rel_oid, array_oid = row.values_at(:oid, :typrelid, :typarray).map{|i| i.to_i}
+          parser_opts[:oid], rel_oid, array_oid = row.values_at(:oid, :typrelid, :typarray).map(&:to_i)
 
           # Get column names and oids for each of the members of the composite type.
           res = from(:pg_attribute).
@@ -485,7 +490,7 @@ module Sequel
             PGArray.register(array_type_name, :oid=>array_oid, :converter=>parser, :type_procs=>@conversion_procs, :scalar_typecast=>schema_type_symbol)
           end
 
-          @row_types[db_type] = opts.merge(:parser=>parser)
+          @row_types[literal(db_type)] = opts.merge(:parser=>parser, :type=>db_type)
           @row_schema_types[schema_type_string] = schema_type_symbol 
           @schema_type_classes[schema_type_symbol] = ROW_TYPE_CLASSES
           @row_type_method_module.class_eval do
@@ -496,6 +501,7 @@ module Sequel
             private meth
           end
 
+          conversion_procs_updated
           nil
         end
 
@@ -504,8 +510,8 @@ module Sequel
         def reset_conversion_procs
           procs = super
 
-          row_types.each do |db_type, opts|
-            register_row_type(db_type, opts)
+          row_types.values.each do |opts|
+            register_row_type(opts[:type], opts)
           end
 
           procs
@@ -515,7 +521,7 @@ module Sequel
         # In general, the given database type should already be registered,
         # but if obj is an array, this will handled unregistered types.
         def row_type(db_type, obj)
-          (type_hash = @row_types[db_type]) &&
+          (type_hash = @row_types[literal(db_type)]) &&
             (parser = type_hash[:parser])
 
           case obj
