@@ -1,17 +1,26 @@
-# The query extension adds Sequel::Dataset#query which allows
+# frozen-string-literal: true
+#
+# The query extension adds a query method which allows
 # a different way to construct queries instead of the usual
-# method chaining.  See Sequel::Dataset#query for details.
+# method chaining:
+#
+#   dataset = DB[:items].query do
+#     select :x, :y, :z
+#     filter{(x > 1) & (y > 2)}
+#     reverse :z
+#   end
 #
 # You can load this extension into specific datasets:
 #
 #   ds = DB[:table]
-#   ds.extension(:query)
+#   ds = ds.extension(:query)
 #
 # Or you can load it into all of a database's datasets, which
 # is probably the desired behavior if you are using this extension:
 #
 #   DB.extension(:query)
 
+#
 module Sequel
   module DatabaseQuery
     def self.extended(db)
@@ -25,6 +34,8 @@ module Sequel
   end
 
   module DatasetQuery
+    Dataset.def_mutation_method(:query, :module=>self)
+
     # Translates a query block into a dataset. Query blocks are an
     # alternative to Sequel's usual method chaining, by using
     # instance_eval with a proxy object:
