@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+#
 # This adds the following dataset methods:
 #
 # []= :: filter with the first argument, update with the second
@@ -19,6 +21,7 @@
 #
 #   DB.extension(:sequel_3_dataset_methods)
 
+#
 module Sequel
   module Sequel3DatasetMethods
     COMMA = Dataset::COMMA
@@ -106,7 +109,7 @@ module Sequel
     def to_csv(include_column_titles = true)
       n = naked
       cols = n.columns
-      csv = ''
+      csv = String.new
       csv << "#{cols.join(COMMA)}\r\n" if include_column_titles
       n.each{|r| csv << "#{cols.collect{|c| r[c]}.join(COMMA)}\r\n"}
       csv
